@@ -5,19 +5,19 @@
 modname = 'cubicweb_jsonschema'
 distname = 'cubicweb-jsonschema'
 
-numversion = (0, 1, 5)
+numversion = (0, 2, 6)
 version = '.'.join(str(num) for num in numversion)
 
 license = 'LGPL'
 author = 'LOGILAB S.A. (Paris, FRANCE)'
 author_email = 'contact@logilab.fr'
 description = 'JSON Schema for CubicWeb'
-web = 'http://www.cubicweb.org/project/%s' % distname
+web = 'https://www.cubicweb.org/project/%s' % distname
 
 __depends__ = {
-    'cubicweb[pyramid]': '>= 3.28.2',
-    'six': '>= 1.4.0',
-    'jsonschema': None,
+    'cubicweb[pyramid]': '>= 3.26.19',
+    'six': '>= 1.12.0',
+    'iso8601': None,
     'jsl': None,
     'pyramid': '>= 1.10.0',
 }
