@@ -15,10 +15,13 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """cubicweb-jsonschema Pyramid views for the JSON API of entities."""
-import json
 
 from pyramid import httpexceptions
 from pyramid.renderers import render
+from pyramid.traversal import (
+    find_resource,
+    find_root,
+)
 from pyramid.view import view_config
 
 from cubicweb import (
@@ -26,28 +29,25 @@ from cubicweb import (
     ValidationError,
 )
 from ..resources import (
-    RelationshipResource,
-)
-from ..resources.entities import (
     EntityResource,
     ETypeResource,
     RelatedEntitiesResource,
     RelatedEntityResource,
+    RelationshipResource,
     RootResource,
 )
 from . import (
     describedby,
     entity_from_context,
-    jsonapi_error,
-    JSONBadRequest,
     json_config,
+    json_problem,
     jsonschema_adapter,
     LOG,
 )
 
 
 @view_config(
-    route_name='cubicweb-jsonschema.entities.root',
+    route_name='cubicweb-jsonschema.entities',
     context=RootResource,
     request_method='GET',
     decorator=[describedby],
@@ -60,7 +60,7 @@ def get_root(context, request):
 
 
 @json_config(
-    route_name='cwentities',
+    route_name='cubicweb-jsonschema.entities',
     context=ETypeResource,
     request_method='GET',
     decorator=[describedby],
@@ -88,7 +88,7 @@ def allow_for_entity(entity):
 
 
 @json_config(
-    route_name='cwentities',
+    route_name='cubicweb-jsonschema.entities',
     context=EntityResource,
     request_method='GET',
     decorator=[entity_from_context, describedby],
@@ -101,7 +101,7 @@ def get_entity(context, request):
 
 
 @json_config(
-    route_name='cwentities',
+    route_name='cubicweb-jsonschema.entities',
     context=ETypeResource,
     request_method='POST',
 )
@@ -110,7 +110,6 @@ def create_entity(context, request):
     # TODO In case of validation errors, it'd be better to give a JSON Schema
     # entry as a "pointer", would require selection context to be an
     # ETypeSchemaResource.
-    etype = context.etype
     adapter = jsonschema_adapter(request.cw_request, etype=context.etype)
     instance = request.json_body
     entity = adapter.create_entity(instance)
@@ -118,9 +117,8 @@ def create_entity(context, request):
     LOG.info('created %s', entity)
 
     value = entity.cw_adapt_to('IJSONSchema').serialize()
-    location = request.route_url('cwentities',
-                                 etype=etype,
-                                 traverse=str(entity.eid))
+    entity_resource = context[entity.eid]
+    location = request.resource_url(entity_resource)
     raise httpexceptions.HTTPCreated(
         location=location,
         content_type='application/json; charset=UTF-8',
@@ -129,7 +127,7 @@ def create_entity(context, request):
 
 
 @json_config(
-    route_name='cwentities',
+    route_name='cubicweb-jsonschema.entities',
     context=EntityResource,
     request_method='PUT',
     decorator=[entity_from_context],
@@ -181,7 +179,7 @@ def allow_for_entity_relation(entity, rtype, role):
 
 
 @json_config(
-    route_name='cwentities',
+    route_name='cubicweb-jsonschema.entities',
     context=RelatedEntitiesResource,
     request_method='GET',
     decorator=[describedby],
@@ -200,7 +198,7 @@ def get_related_entities(context, request):
 
 
 @json_config(
-    route_name='cwentities',
+    route_name='cubicweb-jsonschema.entities',
     context=RelatedEntitiesResource,
     request_method='POST',
 )
@@ -226,10 +224,10 @@ def post_related_entities(context, request):
 
 
 @json_config(
-    route_name='cwentities',
+    route_name='cubicweb-jsonschema.entities',
     context=RelatedEntityResource,
     request_method='GET',
-    # TODO: describedby
+    decorator=[describedby],
 )
 def get_related_entity(context, request):
     """Return a JSON document of an entity target of a relationship."""
@@ -242,7 +240,7 @@ def get_related_entity(context, request):
 
 
 @json_config(
-    route_name='cwentities',
+    route_name='cubicweb-jsonschema.entities',
     context=RelatedEntityResource,
     request_method='PUT',
 )
@@ -284,7 +282,7 @@ def delete_relation(context, request):
 
 
 @json_config(
-    route_name='cwentities',
+    route_name='cubicweb-jsonschema.entities',
     context=RelationshipResource,
     request_method='POST',
     decorator=[entity_from_context],
@@ -303,9 +301,10 @@ def create_related_entity(context, request):
     related_adapted = jsonschema_adapter(
         request.cw_request, entity=related, rtype=rtype, role=neg_role(role))
     value = related_adapted.serialize()
-    location = request.route_url('cwentities',
-                                 etype=related.cw_etype,
-                                 traverse=str(related.eid))
+    root_resource = find_root(context)
+    entity_resource = find_resource(root_resource,
+                                    (related.cw_etype, related.eid))
+    location = request.resource_url(entity_resource)
     raise httpexceptions.HTTPCreated(
         location=location,
         content_type='application/json; charset=UTF-8',
@@ -314,7 +313,7 @@ def create_related_entity(context, request):
 
 
 @json_config(
-    route_name='cwentities',
+    route_name='cubicweb-jsonschema.entities',
     context=ValidationError,
 )
 def validation_failed(exc, request):
@@ -323,13 +322,23 @@ def validation_failed(exc, request):
     _ = request.cw_request._
     request.cw_cnx.rollback()
     exc.translate(_)
-    errors = [jsonapi_error(status=422, details=value, pointer=rolename)
-              for rolename, value in exc.errors.items()]
-    return JSONBadRequest(*errors)
+    invalid_params = []
+    detail = []
+    for rolename, value in exc.errors.items():
+        if rolename:
+            invalid_params.append({'name': rolename, 'reason': value})
+        else:
+            detail.append(value)
+    kwargs = {}
+    if detail:
+        kwargs['detail'] = '\n'.join(detail)
+    if invalid_params:
+        kwargs['invalid-params'] = invalid_params
+    return json_problem(status=422, **kwargs)
 
 
 @json_config(
-    route_name='cwentities',
+    route_name='cubicweb-jsonschema.entities',
     context=Exception,
 )
 def generic_error(exc, request):
@@ -343,28 +352,28 @@ def generic_error(exc, request):
     # be catched, including the standard pyramid return exceptions.
     # In this case, just return it
     if isinstance(exc, httpexceptions.HTTPException):
-        if exc.content_type != 'application/json':
-            exc.content_type = 'application/json'
-            body = json.dumps({'message': exc.detail}).encode('utf-8')
-            exc.body = body
         return exc
-    request.cw_cnx.rollback()
+    if request.cw_cnx is not None:
+        request.cw_cnx.rollback()
     LOG.exception('exception occurred while processing %s', request)
-    return JSONBadRequest(jsonapi_error(status=400, details=str(exc)))
+    kwargs = {}
+    detail = str(exc)
+    if detail:
+        kwargs['detail'] = detail
+    return json_problem(status=500, **kwargs)
 
 
 def includeme(config):
     config.include('cubicweb.pyramid.predicates')
     config.include('..predicates')
-    config.add_route('cubicweb-jsonschema.entities.root', '/',
-                     factory=RootResource,
-                     strict_accept='application/json')
     config.add_route('delete_entity', '/{etype}/*traverse',
                      factory=ETypeResource.from_match('etype'),
                      request_method='DELETE',
                      match_is_etype='etype')
-    config.add_route('cwentities', '/{etype}/*traverse',
-                     factory=ETypeResource.from_match('etype'),
-                     strict_accept='application/json',
-                     match_is_etype='etype')
+    config.add_route(
+        'cubicweb-jsonschema.entities',
+        '*traverse',
+        factory=RootResource,
+        strict_accept='application/json',
+    )
     config.scan(__name__)
