@@ -18,7 +18,12 @@
 from logilab.common.registry import objectify_predicate
 
 from cubicweb.predicates import match_kwargs, non_final_entity
-from cubicweb.view import Adapter, EntityAdapter
+try:
+    # 3.28+
+    from cubicweb.entity import Adapter, EntityAdapter
+except ImportError:
+    # 3.26 and 3.27
+    from cubicweb.view import Adapter, EntityAdapter
 
 from cubicweb_jsonschema import (
     VIEW_ROLE,
